// Copyright 2009 the Sputnik authors.  All rights reserved.
// This code is governed by the BSD license found in the LICENSE file.

/**
 * Single and Multi line comments are used together
 *
 * @path ch07/7.4/S7.4_A4_T7.js
 * @description Insert Multi line comment into Single line comments
 */

/*CHECK#1*/

// var /* 
// x 
// =
// 1*/

