// Binary: cache/js-dbg-64-58c9dd4f0c1f-linux
// Flags:
//
var o = {}
Object.defineProperty(o, "x", {get: undefined, set: function() { Object.defineProperty(o, "x", {set: function() { }}); }, configurable: true});
o.a = 0;
o.x = 0;
